@@ -1,15 +1,11 @@
+from datetime import timedelta
+
 from plain.paginator import Paginator
+from plain.urls import reverse
 from plain.utils import timezone
 
 
-def url(viewname, *args, **kwargs):
-    # A modified reverse that lets you pass args directly, excluding urlconf
-    from plain.urls import reverse
-
-    return reverse(viewname, args=args, kwargs=kwargs)
-
-
-def asset(url_path):
+def asset(url_path: str) -> str:
     # An explicit callable we can control, but also delay the import of asset.urls->views->templates
     # for circular import reasons
     from plain.assets.urls import get_asset_url
@@ -19,8 +15,9 @@ def asset(url_path):
 
 default_globals = {
     "asset": asset,
-    "url": url,
+    "url": reverse,
     "Paginator": Paginator,
     "now": timezone.now,
+    "timedelta": timedelta,
     "localtime": timezone.localtime,
 }
